﻿
using System;

namespace WindowsFormsApplication1
{
    class Matrix
    {
        public double[,] inv;
        double[,] a;
        int order;

        public Matrix(int maxOrder)
        {
            a = new double[maxOrder, maxOrder];
            inv = new double[maxOrder, maxOrder];
            order = maxOrder;
        }

        public void InitMatrix(double[,] aIn)
        {
            int i, j;
            for (i = 0; i < order; i++)
            {
                for (j = 0; j < order; j++)
                {
                    a[i, j] = aIn[i, j];
                }
            }
        }

        public double[,] Mult(double[,] a, double[,] b, int xA, int yA, int xB, int yB)
        {
            double[,] c = new double[yA, xA];
            for (int x = 0; x < xB; x++)
            {
                for (int y = 0; y < yA; y++)
                {
                    c[y, x] = 0;
                    for (int k = 0; k < xA; k++)
                        c[y, x] = c[y, x] + a[y, k] * b[k, x];
                }
            }
            return c;
        }

        public double[] Mult(double[,] a, double[] b)
        {
            double[] c = new double[order];
            for (int y = 0; y < order; y++)
            {
                c[y] = 0;
                for (int k = 0; k < order; k++)
                    c[y] = c[y] + a[y, k] * b[k];
            }
            return c;
        }


        double CalcDet(double[,] m, int iOrder)
        {
            int i, j, k;
            double sign = 1.0;
            double dm = 0;
            if (iOrder <= 2)
            {
                return m[0, 0] * m[1, 1] - m[1, 0] * m[0, 1];
            }
            else
            {
                double[,] mm = new double[iOrder - 1, iOrder - 1]; // temporary matrix
                for (i = 0; i < iOrder; i++) // index for the element of the top row
                {
                    for (j = 0; j < iOrder - 1; j++) // index that runs down from second row to n
                    {
                        for (k = 0; k < iOrder - 1; k++) // index to copy the elements into a sub matrix
                        {
                            if (j < i)
                                mm[j, k] = m[j, k + 1];
                            else
                                mm[j, k] = m[j + 1, k + 1];
                        }
                    }
                    dm = dm + sign * CalcDet(mm, iOrder - 1) * m[i, 0];
                    sign = -sign;
                }
                return dm;
            }
        }

        public void Invert()
        {
            double det = CalcDet(a, order);

            int x, y, j, k;
            double signy = 1.0;
            double signx = 1.0;
            if (det != 0)
            {
                if (order == 2)
                {
                    inv[0, 0] = a[1, 1] / det;
                    inv[0, 1] = -a[0, 1] / det;
                    inv[1, 0] = -a[0, 1] / det;
                    inv[1, 1] = a[0, 0] / det;
                }
                else
                {
                    double[,] mm = new double[order - 1, order - 1]; // temporary matrix
                    for (x = 0; x < order; x++)
                    {
                        signy = 1.0;
                        for (y = 0; y < order; y++) // index for the element of the top row
                        {
                            for (j = 0; j < order - 1; j++) // index that runs down from second row to n
                            {
                                for (k = 0; k < order - 1; k++) // index to copy the elements into a sub matrix
                                {
                                    if (j < y)
                                    {
                                        if (k < x)
                                            mm[j, k] = a[j, k];
                                        else
                                            mm[j, k] = a[j, k + 1];
                                    }
                                    else
                                    {
                                        if (k < x)
                                            mm[j, k] = a[j + 1, k];
                                        else
                                            mm[j, k] = a[j + 1, k + 1];
                                    }
                                }
                            }
                            inv[x, y] = signy * signx * CalcDet(mm, order - 1) / det;
                            signy = -signy;
                        }
                        signx = -signx;
                    }
                }
            }
        }
    }

}
